#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <stdint.h>

#define ATY_RADEON_LCD_ON       0x00000001
#define ATY_RADEON_CRT_ON       0x00000002

#define FBIO_RADEON_GET_MIRROR  _IOR('@', 3, uint32_t)
#define FBIO_RADEON_SET_MIRROR  _IOW('@', 4, uint32_t)


static void
usage(void)
{
        printf("syntax: m6mirror [crt:0|1] [lcd:0|1]\n");
}

int main(int ac, char **av)
{
        int fd, rc, i;
        unsigned long value, orig;

        printf("ATI Radeon M6 mirror tool, v0.1\n");
        fd = open("/dev/fb0", 0);
        if (fd < 0) {
                perror("open /dev/fb0");
                exit(1);
        }
        rc = ioctl(fd, FBIO_RADEON_GET_MIRROR, &value);
        if (rc != 0) {
                printf("error %d getting mirror value\n", rc);
                goto bail;
        }
        printf("Mirror is currently: lcd: %s, crt: %s\n",
                (value & ATY_RADEON_LCD_ON) ? "on" : "off",
                (value & ATY_RADEON_CRT_ON) ? "on" : "off");
        orig = value;
        for(i=1; i<ac; i++) {
                if (!strncmp(av[i], "lcd:", 4)) {
                        if (atoi((av[i])+4))
                                value |= ATY_RADEON_LCD_ON;
                        else
                                value &= ~ATY_RADEON_LCD_ON;
                } else if (!strncmp(av[i], "crt:", 4)) {
                        if (atoi((av[i])+4))
                                value |= ATY_RADEON_CRT_ON;
                        else
                                value &= ~ATY_RADEON_CRT_ON;
                } else {
                        usage();
                        rc = -EINVAL;
                        goto bail;
                }
        }
        if (orig != value) {
                rc = ioctl(fd, FBIO_RADEON_SET_MIRROR, &value);
                if (rc != 0) {
                        printf("error %d setting mirror value\n", rc);
                        exit(rc);
                }
                printf("Mirror is now: lcd: %s, crt: %s\n",
                        (value & ATY_RADEON_LCD_ON) ? "on" : "off",
                        (value & ATY_RADEON_CRT_ON) ? "on" : "off");
        }
bail:
        close(fd);
        exit(rc);
}

